# Copyright (c) 2012 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

#
# Compute path to compiler
#
OSNAME:=$(shell python $(NACL_SDK_ROOT)/tools/getos.py)

ifeq (win,$(OSNAME))
# libmono was compiled in a posix environment (NaCl) so
# use forward slashes, Windows will cope with the paths.
NACL_SDK_ROOT:=$(subst \,/,$(NACL_SDK_ROOT))
NACL_MONO_ROOT:=$(subst \,/,$(NACL_MONO_ROOT))
endif

ARCH:=$(shell python $(NACL_SDK_ROOT)/tools/getos.py --arch)
TC_PATH:=$(abspath $(NACL_SDK_ROOT)/toolchain/$(OSNAME)_x86_glibc)

#
# Verify we have a valid NACL_SDK_ROOT
#
ifeq (,$(wildcard $(TC_PATH)))
$(warning No valid NACL_SDK_ROOT at $(NACL_SDK_ROOT))
ifeq ($(origin NACL_SDK_ROOT), 'file')
$(error Override the default value via enviornment variable, or command-line.)
else
$(error Fix the NACL_SDK_ROOT specified in the environment or command-line.)
endif
endif

# Alias for C compiler
CC:=$(TC_PATH)/bin/i686-nacl-gcc

#
# Disable DOS PATH warning when using Cygwin based tools Windows
#
CYGWIN ?= nodosfilewarning
export CYGWIN

# Default target is everything
all : $(PROJECT)_x86_32.nexe $(PROJECT)_x86_64.nexe $(PROJECT).nmf $(CS_OUTPUT) mscorlib.dll $(EXTRA_TARGETS)

ifeq (win,$(OSNAME))
RM:=del /s /f /q
else
RM:=rm -rf
endif

.PHONY: clean
clean :
	$(RM) *.nexe *.exe *.nacllog *.log *.nmf lib32 lib64 mscorlib.dll $(EXTRA_TARGETS)

# Define compile and link rule for 32 bit (-m32) nexe
$(PROJECT)_x86_32.nexe : $(CC_SOURCES) $(THIS_MAKE)
	$(CC) -o $@ $(CC_SOURCES) -m32 -O0 -g $(CFLAGS) -DPROJECT_NAME="\"$(PROJECT)\"" $(LDFLAGS) -DCS_CLASS_NAME="\"$(CS_CLASS_NAME)\""

# Define compile and link rule for 64 bit (-m64) nexe
$(PROJECT)_x86_64.nexe : $(CC_SOURCES) $(THIS_MAKE)
	$(CC) -o $@ $(CC_SOURCES) -m64 -O0 -g $(CFLAGS) -DPROJECT_NAME="\"$(PROJECT)\"" $(LDFLAGS) -DCS_CLASS_NAME="\"$(CS_CLASS_NAME)\""

#
# NMF Manifiest generation
#
# Use the python script create_nmf to scan the binaries for dependencies using
# objdump.  Pass in the (-L) paths to the default library toolchains so that we
# can find those libraries and have it automatically copy the files (-s) to
# the target directory for us.
NMF:=python $(NACL_SDK_ROOT)/tools/create_nmf.py
NMF_ARGS:=-D $(TC_PATH)/x86_64-nacl/bin/objdump
NMF_PATHS:=-L $(TC_PATH)/x86_64-nacl/lib32 -L $(TC_PATH)/x86_64-nacl/lib -L $(NACL_MONO_ROOT)/lib32 -L $(NACL_MONO_ROOT)/lib -L outlib32 -L outlib64

$(PROJECT).nmf : $(PROJECT)_x86_64.nexe $(PROJECT)_x86_32.nexe $(EXTRA_TARGETS)
	echo $(NMF) $(NMF_ARGS) -s . -o $@ $(NMF_PATHS) $^ $(EXTRA_NMF_ARGS)
	$(NMF) $(NMF_ARGS) -s . -o $@ $(NMF_PATHS) $^ $(EXTRA_NMF_ARGS)


# On Linux/Mac, use sel_ldr with Mono to compile the assembly
ifeq (x86_64,$(ARCH))
SEL_LDR:=$(NACL_SDK_ROOT)/tools/sel_ldr_x86_64
IRT:=$(NACL_SDK_ROOT)/tools/irt_core_x86_64.nexe
RTLD:=$(TC_PATH)/x86_64-nacl/lib/runnable-ld.so
LIBDIR:=$(TC_PATH)/x86_64-nacl/lib:$(NACL_MONO_ROOT)/lib
MONO:=$(NACL_MONO_ROOT)/bin/x86_64-nacl-mono
else
SEL_LDR:=$(NACL_SDK_ROOT)/tools/sel_ldr_x86_32
IRT:=$(NACL_SDK_ROOT)/tools/irt_core_x86_32.nexe
RTLD:=$(TC_PATH)/x86_64-nacl/lib32/runnable-ld.so
LIBDIR:=$(TC_PATH)/x86_64-nacl/lib32:$(NACL_MONO_ROOT)/lib32
MONO:=$(NACL_MONO_ROOT)/bin/i686-nacl-mono
endif
GMCS:=$(NACL_MONO_ROOT)/lib/mono/4.0/mcs.exe -sdk:2

$(CS_OUTPUT) : $(CS_SOURCES)
        # Either Mono or sel_ldr doesn't like PWD to have <drive-letter>: in it
        # so strip it.  It is set in make.bat on Windows and not set otherwise.
	$(SEL_LDR) -E "NACL_PWD=$(subst $(CURRENTDRIVE),,$(CURDIR))" -E "MONO_PATH=$(NACL_MONO_ROOT)/lib/mono/4.0" -a -S -B $(IRT) -l $(CS_OUTPUT).nacllog -- $(RTLD) --library-path $(LIBDIR) $(MONO) $(GMCS) $(CS_SOURCES) -out:$@

ifeq (win,$(OSNAME))
# pepper_19 windows left .exe off sel_ldr executables
$(CS_SOURCES) : $(SEL_LDR).exe
$(SEL_LDR).exe :
	copy $(subst /,\,$(SEL_LDR)) $(subst /,\,$(SEL_LDR)).exe
# Windows copy requires backslash paths
mscorlib.dll : $(NACL_MONO_ROOT)/lib/mono/4.0/mscorlib.dll
	copy $(subst /,\,$^) $@
else
mscorlib.dll : $(NACL_MONO_ROOT)/lib/mono/4.0/mscorlib.dll
	cp $^ $@
endif
