/* Copyright (c) 2012 The Chromium Authors. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be
 * found in the LICENSE file.
 */

#include <string.h>

#include "ppapi/c/ppb_core.h"
#include "ppapi/c/ppb_messaging.h"
#include "ppapi/c/ppb_var.h"

#include "my_assert.h"
#include "post_message.h"

static const PPB_Core* g_core;
static const PPB_Messaging* g_messaging;
static const PPB_Var* g_var;
static PP_Instance g_instance;

void initialize_post_message(PP_Instance instance, const void* messaging,
                             const void* core, const void* var) {
  assert(messaging);
  g_instance = instance;
  g_messaging = (const PPB_Messaging*)messaging;
  g_core = (const PPB_Core*)core;
  g_var = (const PPB_Var*)var;
}

void post_message_main_thread(void* data, int32_t result) {
  char* msg = (char*)data;
  struct PP_Var msg_var = g_var->VarFromUtf8(msg, strlen(msg));
  g_messaging->PostMessage(g_instance, msg_var);
  g_var->Release(msg_var);
  free((char*)msg);
}

void post_message(const char *msg) {
  char* new_msg = strdup(msg);
  if (g_core->IsMainThread()) {
    post_message_main_thread((void*)new_msg, 0);
  } else {
    struct PP_CompletionCallback cb =
        PP_MakeCompletionCallback(post_message_main_thread, (void*)new_msg);
    g_core->CallOnMainThread(0, cb, 0);
  }
  
}

